@@ -20,6 +20,8 @@ nocaselist = import_installed('nocaselist')
 from nocaselist import NocaseList as _NocaseList  # noqa: E402
 # pylint: enable=wrong-import-position, wrong-import-order, invalid-name
 
+# pylint: disable=use-dict-literal
+
 # Controls whether the tests are run against a standard dict instead.
 TEST_AGAINST_LIST = bool(os.getenv('TEST_AGAINST_LIST'))
 
@@ -40,6 +42,17 @@ NocaseList = list if TEST_AGAINST_LIST else _NocaseList
 DICT_PRESERVES_ORDER = sys.version_info[0:2] >= (3, 7)
 
 
+def ensure_unicode(value):
+    """
+    Return the value as a unicode string.
+    """
+    if value is None:
+        return None
+    if isinstance(value, six.text_type):
+        return value
+    return value.decode('utf-8')
+
+
 def assert_equal(list1, list2, verify_order=True):
     """
     Assert that list1 is equal to list2.
@@ -55,7 +68,8 @@ def assert_equal(list1, list2, verify_order=True):
     if verify_order:
         assert list1_lst == list2_lst
     else:
-        assert sorted(list1_lst) == sorted(list2_lst)
+        assert sorted(list1_lst, key=ensure_unicode) == \
+            sorted(list2_lst, key=ensure_unicode)
 
     # Check consistency of list1 between its internal casefolded list and
     # its external list
@@ -138,9 +152,9 @@ TESTCASES_NOCASELIST_INIT = [
     (
         "List from list as positional arg",
         dict(
-            init_args=(['Dog', 'Cat'],),
+            init_args=([u'Dog', b'Cat'],),
             init_kwargs={},
-            exp_list=['Dog', 'Cat'],
+            exp_list=[u'Dog', b'Cat'],
             verify_order=True,
         ),
         None, None, True
@@ -148,9 +162,9 @@ TESTCASES_NOCASELIST_INIT = [
     (
         "List from tuple as positional arg",
         dict(
-            init_args=(('Dog', 'Cat'),),
+            init_args=((u'Dog', b'Cat'),),
             init_kwargs={},
-            exp_list=['Dog', 'Cat'],
+            exp_list=[u'Dog', b'Cat'],
             verify_order=True,
         ),
         None, None, True
@@ -158,9 +172,9 @@ TESTCASES_NOCASELIST_INIT = [
     (
         "List from dict as positional arg (uses only the keys)",
         dict(
-            init_args=({'Dog': 'Kitten', 'Cat': 'Budgie'},),
+            init_args=({u'Dog': 'Kitten', b'Cat': 'Budgie'},),
             init_kwargs={},
-            exp_list=['Dog', 'Cat'],
+            exp_list=[u'Dog', b'Cat'],
             verify_order=DICT_PRESERVES_ORDER,
         ),
         None, None, True
@@ -323,10 +337,19 @@ TESTCASES_NOCASELIST_GETITEM = [
         TypeError, None, True
     ),
     (
-        "Empty list, with string as index (invalid type)",
+        "Empty list, with unicode string as index (invalid type)",
         dict(
             nclist=NocaseList(),
-            index='',
+            index=u'',
+            exp_value=None,
+        ),
+        TypeError, None, True
+    ),
+    (
+        "Empty list, with byte string as index (invalid type)",
+        dict(
+            nclist=NocaseList(),
+            index=b'',
             exp_value=None,
         ),
         TypeError, None, True
@@ -345,7 +368,7 @@ TESTCASES_NOCASELIST_GETITEM = [
     (
         "List with two items, with None as index (invalid type)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=None,
             exp_value=None,
         ),
@@ -354,8 +377,8 @@ TESTCASES_NOCASELIST_GETITEM = [
     (
         "List with two items, with string as index (invalid type)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            index='',
+            nclist=NocaseList([u'Dog', b'Cat']),
+            index=u'',
             exp_value=None,
         ),
         TypeError, None, True
@@ -363,7 +386,7 @@ TESTCASES_NOCASELIST_GETITEM = [
     (
         "List with two items, with non-existing index 2 (out of range)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=2,
             exp_value=None,
         ),
@@ -372,7 +395,7 @@ TESTCASES_NOCASELIST_GETITEM = [
     (
         "List with two items, with index 0",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=0,
             exp_value='Dog',
         ),
@@ -381,18 +404,18 @@ TESTCASES_NOCASELIST_GETITEM = [
     (
         "List with two items, with index 1",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=1,
-            exp_value='Cat',
+            exp_value=b'Cat',
         ),
         None, None, True
     ),
     (
         "List with two items, with index -1",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=-1,
-            exp_value='Cat',
+            exp_value=b'Cat',
         ),
         None, None, True
     ),
@@ -445,10 +468,20 @@ TESTCASES_NOCASELIST_SETITEM = [
         TypeError, None, True
     ),
     (
-        "Empty list, with string as index (invalid type)",
+        "Empty list, with unicode string as index (invalid type)",
         dict(
             nclist=NocaseList(),
-            index='',
+            index=u'',
+            value=None,
+            exp_nclist=None,
+        ),
+        TypeError, None, True
+    ),
+    (
+        "Empty list, with byte string as index (invalid type)",
+        dict(
+            nclist=NocaseList(),
+            index=b'',
             value=None,
             exp_nclist=None,
         ),
@@ -469,49 +502,49 @@ TESTCASES_NOCASELIST_SETITEM = [
     (
         "List with two items, with None as index (invalid type)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             index=None,
-            value='Kitten',
+            value=u'Kitten',
             exp_nclist=None,
         ),
         TypeError, None, True
     ),
     (
-        "List with two items, with index 0",
+        "List with two items, setting unicode string at index 0",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', u'Cat']),
             index=0,
-            value='Newbie',
-            exp_nclist=NocaseList(['Newbie', 'Cat']),
+            value=u'Newbie',
+            exp_nclist=NocaseList([u'Newbie', u'Cat']),
         ),
         None, None, True
     ),
     (
-        "List with two items, with index 1",
+        "List with two items, setting byte string at index 1",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', u'Cat']),
             index=1,
-            value='Newbie',
-            exp_nclist=NocaseList(['Dog', 'Newbie']),
+            value=b'Newbie',
+            exp_nclist=NocaseList([u'Dog', b'Newbie']),
         ),
         None, None, True
     ),
     (
-        "List with two items, with index -1",
+        "List with two items, setting string at index -1",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', u'Cat']),
             index=-1,
-            value='Newbie',
-            exp_nclist=NocaseList(['Dog', 'Newbie']),
+            value=u'Newbie',
+            exp_nclist=NocaseList([u'Dog', u'Newbie']),
         ),
         None, None, True
     ),
     (
-        "List with two items, with non-existing index 2 (out of range)",
+        "List with two items, setting string at index 2 (out of range)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', u'Cat']),
             index=2,
-            value='Newbie',
+            value=u'Newbie',
             exp_nclist=None,
         ),
         IndexError, None, True
@@ -752,19 +785,39 @@ TESTCASES_NOCASELIST_CONTAINS = [
         None if TEST_AGAINST_LIST else AttributeError, None, True
     ),
     (
-        "Empty list, with non-existing empty value (not found)",
+        "Empty list, with non-existing empty unicode string value (not found)",
         dict(
             nclist=NocaseList(),
-            value='',
+            value=u'',
             exp_result=False,
         ),
         None, None, True
     ),
     (
-        "Empty list, with non-existing non-empty value (not found)",
+        "Empty list, with non-existing empty byte string value (not found)",
         dict(
             nclist=NocaseList(),
-            value='Dog',
+            value=b'',
+            exp_result=False,
+        ),
+        None, None, True
+    ),
+    (
+        "Empty list, with non-existing non-empty unicode string value "
+        "(not found)",
+        dict(
+            nclist=NocaseList(),
+            value=u'Dog',
+            exp_result=False,
+        ),
+        None, None, True
+    ),
+    (
+        "Empty list, with non-existing non-empty byte string value "
+        "(not found)",
+        dict(
+            nclist=NocaseList(),
+            value=b'Dog',
             exp_result=False,
         ),
         None, None, True
@@ -774,7 +827,7 @@ TESTCASES_NOCASELIST_CONTAINS = [
     (
         "List with two items, with None as value (no casefold method)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             value=None,
             exp_result=False,
         ),
@@ -783,62 +836,128 @@ TESTCASES_NOCASELIST_CONTAINS = [
     (
         "List with two items, with integer value 0 (no casefold method)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
+            nclist=NocaseList([u'Dog', b'Cat']),
             value=0,
             exp_result=False if TEST_AGAINST_LIST else None,
         ),
         None if TEST_AGAINST_LIST else AttributeError, None, True
     ),
     (
-        "List with two items, with non-existing empty value (not found)",
+        "List with two items, with non-existing empty unicode string value "
+        "(not found)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='',
+            nclist=NocaseList([u'Dog', b'Cat']),
+            value=u'',
             exp_result=False,
         ),
         None, None, True
     ),
     (
-        "List with two items, with non-existing non-empty value (not found)",
+        "List with two items, with non-existing empty byte string value "
+        "(not found)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='invalid',
+            nclist=NocaseList([u'Dog', b'Cat']),
+            value=b'',
             exp_result=False,
         ),
         None, None, True
     ),
     (
-        "List with two items, with existing value in original case",
+        "List with two items, with non-existing non-empty unicode string value "
+        "(not found)",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='Dog',
+            nclist=NocaseList([u'Dog', b'Cat']),
+            value=u'invalid',
+            exp_result=False,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with non-existing non-empty byte string value "
+        "(not found)",
+        dict(
+            nclist=NocaseList([u'Dog', b'Cat']),
+            value=b'invalid',
+            exp_result=False,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with existing unicode string value in original "
+        "case",
+        dict(
+            nclist=NocaseList([u'Dog', b'Cat']),
+            value=u'Dog',
             exp_result=True,
         ),
         None, None, True
     ),
     (
-        "List with two items, with existing value in non-original upper case",
+        "List with two items, with existing byte string value in original "
+        "case",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='DOG',
+            nclist=NocaseList([b'Dog', u'Cat']),
+            value=b'Dog',
+            exp_result=True,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with existing unicode string value in "
+        "non-original upper case",
+        dict(
+            nclist=NocaseList([u'Dog', u'Cat']),
+            value=u'DOG',
             exp_result=not TEST_AGAINST_LIST,
         ),
         None, None, True
     ),
     (
-        "List with two items, with existing value in non-original lower case",
+        "List with two items, with existing byte string value in "
+        "non-original upper case",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='dog',
+            nclist=NocaseList([b'Dog', u'Cat']),
+            value=b'DOG',
             exp_result=not TEST_AGAINST_LIST,
         ),
         None, None, True
     ),
     (
-        "List with two items, with existing value in non-original mixed case",
+        "List with two items, with existing unicode string value in "
+        "non-original lower case",
         dict(
-            nclist=NocaseList(['Dog', 'Cat']),
-            value='doG',
+            nclist=NocaseList([u'Dog', u'Cat']),
+            value=u'dog',
+            exp_result=not TEST_AGAINST_LIST,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with existing byte string value in "
+        "non-original lower case",
+        dict(
+            nclist=NocaseList([b'Dog', u'Cat']),
+            value=b'dog',
+            exp_result=not TEST_AGAINST_LIST,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with existing unicode string value in "
+        "non-original mixed case",
+        dict(
+            nclist=NocaseList([u'Dog', u'Cat']),
+            value=u'doG',
+            exp_result=not TEST_AGAINST_LIST,
+        ),
+        None, None, True
+    ),
+    (
+        "List with two items, with existing byte string value in "
+        "non-original mixed case",
+        dict(
+            nclist=NocaseList([b'Dog', u'Cat']),
+            value=b'doG',
             exp_result=not TEST_AGAINST_LIST,
         ),
         None, None, True
